@@ -21,6 +21,7 @@ from .autodecode import (
 from .cache import (
     cached_tarfile_samples,
     cached_tarfile_to_samples,
+    maybe_cached_tarfile_to_samples,
     lru_cleanup,
     pipe_cleaner,
 )
@@ -75,5 +76,6 @@ from .tariterators import tarfile_samples, tarfile_to_samples
 from .utils import PipelineStage, repeatedly
 from .writer import ShardWriter, TarWriter, numpy_dumps, torch_dumps
 from .mix import RandomMix, RoundRobin
+from .gopen import gopen, gopen_schemes
 
-__vesion__ = ""
+__version__ = ""
