@@ -11,9 +11,82 @@ from functools import partial
 
 import numpy as np
 
-"""Extensions passed on to the image decoder."""
-image_extensions = "jpg jpeg png ppm pgm pbm pnm".split()
 
+# Obtained with:
+# ```
+# import PIL.Image
+# IMAGE_EXTENSIONS = []
+# PIL.Image.init()
+# for ext, format in PIL.Image.EXTENSION.items():
+#     if format in PIL.Image.OPEN:
+#         IMAGE_EXTENSIONS.append(ext[1:])
+# ```
+"""Extensions passed on to the image decoder."""
+IMAGE_EXTENSIONS = [
+    "blp",
+    "bmp",
+    "dib",
+    "bufr",
+    "cur",
+    "pcx",
+    "dcx",
+    "dds",
+    "ps",
+    "eps",
+    "fit",
+    "fits",
+    "fli",
+    "flc",
+    "ftc",
+    "ftu",
+    "gbr",
+    "gif",
+    "grib",
+    "h5",
+    "hdf",
+    "png",
+    "apng",
+    "jp2",
+    "j2k",
+    "jpc",
+    "jpf",
+    "jpx",
+    "j2c",
+    "icns",
+    "ico",
+    "im",
+    "iim",
+    "tif",
+    "tiff",
+    "jfif",
+    "jpe",
+    "jpg",
+    "jpeg",
+    "mpg",
+    "mpeg",
+    "msp",
+    "pcd",
+    "pxr",
+    "pbm",
+    "pgm",
+    "ppm",
+    "pnm",
+    "psd",
+    "bw",
+    "rgb",
+    "rgba",
+    "sgi",
+    "ras",
+    "tga",
+    "icb",
+    "vda",
+    "vst",
+    "webp",
+    "wmf",
+    "emf",
+    "xbm",
+    "xpm",
+]
 
 ################################################################
 # handle basic datatypes
@@ -199,7 +272,7 @@ class ImageHandler:
 
     """
 
-    def __init__(self, imagespec, extensions=image_extensions):
+    def __init__(self, imagespec, extensions=IMAGE_EXTENSIONS):
         """Create an image handler.
 
         :param imagespec: short string indicating the type of decoding
@@ -227,32 +300,59 @@ class ImageHandler:
             img = PIL.Image.open(stream)
             img.load()
             img = img.convert(mode.upper())
+
         if atype == "pil":
-            return img
-        elif atype == "numpy":
-            result = np.asarray(img)
-            if result.dtype != np.uint8:
-                raise ValueError("ImageHandler: numpy image must be uint8")
-            if etype == "uint8":
-                return result
+            if mode == "l":
+                img = img.convert("L")
+                return img
+            elif mode == "rgb":
+                img = img.convert("RGB")
+                return img
+            elif mode == "rgba":
+                img = img.convert("RGBA")
+                return img
             else:
-                return result.astype("f") / 255.0
+                raise ValueError("Unknown mode: %s" % mode)
+
+        result = result = np.asarray(img)
+
+        if etype == "float":
+            result = result.astype(np.float32) / 255.0
+
+        assert result.ndim in [2, 3], result.shape
+        assert mode in ["l", "rgb", "rgba"], mode
+
+        if mode == "l":
+            if result.ndim == 3:
+                result = np.mean(result[:, :, :3], axis=2)
+        elif mode == "rgb":
+            if result.ndim == 2:
+                result = np.repeat(result[:, :, np.newaxis], 3, axis=2)
+            elif result.shape[2] == 4:
+                result = result[:, :, :3]
+        elif mode == "rgba":
+            if result.ndim == 2:
+                result = np.repeat(result[:, :, np.newaxis], 4, axis=2)
+                result[:, :, 3] = 255
+            elif result.shape[2] == 3:
+                result = np.concatenate([result, 255 * np.ones(result.shape[:2])], axis=2)
+
+        assert atype in ["numpy", "torch"], atype
+
+        if atype == "numpy":
+            return result
         elif atype == "torch":
             import torch
 
-            result = np.asarray(img)
-            if result.dtype != np.uint8:
-                raise ValueError("ImageHandler: torch image must be uint8")
-            if etype == "uint8":
-                result = np.array(result.transpose(2, 0, 1))
-                return torch.tensor(result)
+            if result.ndim == 3:
+                return torch.from_numpy(result.transpose(2, 0, 1))
             else:
-                result = np.array(result.transpose(2, 0, 1))
-                return torch.tensor(result) / 255.0
+                return torch.from_numpy(result)
+
         return None
 
 
-def imagehandler(imagespec, extensions=image_extensions):
+def imagehandler(imagespec, extensions=IMAGE_EXTENSIONS):
     """Create an image handler.
 
     This is just a lower case alias for ImageHander.
@@ -414,9 +514,12 @@ class Decoder:
         result = {}
         assert isinstance(sample, dict), sample
         for k, v in list(sample.items()):
-            if k[0] == "_":
+            if k[0:2] == "__":
                 if isinstance(v, bytes):
-                    v = v.decode("utf-8")
+                    try:
+                        v = v.decode("utf-8")
+                    except:
+                        print(f"Can't decode v of k = {k} as utf-8: v = {v}")
                 result[k] = v
                 continue
             if self.only is not None and k not in self.only:
@@ -429,7 +532,7 @@ class Decoder:
                 else:
                     result[k] = v
             else:
-                assert isinstance(v, bytes)
+                assert isinstance(v, bytes), f"k,v = {k}, {v}"
                 result[k] = self.decode1(k, v)
         return result
 
