@@ -72,7 +72,9 @@ def split_by_node(src, group=None):
 def single_node_only(src, group=None):
     rank, world_size, worker, num_workers = utils.pytorch_worker_info(group=group)
     if world_size > 1:
-        raise ValueError("you need to add an explicit nodesplitter to your input pipeline for multi-node training")
+        raise ValueError(
+            "you need to add an explicit nodesplitter to your input pipeline for multi-node training"
+        )
     for s in src:
         yield s
 
@@ -112,7 +114,9 @@ def non_empty(src):
         yield s
         count += 1
     if count == 0:
-        raise ValueError("pipeline stage received no data at all and this was declared as an error")
+        raise ValueError(
+            "pipeline stage received no data at all and this was declared as an error"
+        )
 
 
 @dataclass
@@ -146,20 +150,24 @@ class MultiShardSample(IterableDataset):
         else:
             with open(fname) as stream:
                 spec = yaml.safe_load(stream)
-        assert set(spec.keys()).issubset(set("prefix datasets buckets".split())), list(spec.keys())
+        assert set(spec.keys()).issubset(set("prefix datasets buckets".split())), list(
+            spec.keys()
+        )
         prefix = expand(spec.get("prefix", ""))
         self.sources = []
         for ds in spec["datasets"]:
-            assert set(ds.keys()).issubset(set("buckets name shards resample choose".split())), list(
-                ds.keys()
-            )
+            assert set(ds.keys()).issubset(
+                set("buckets name shards resample choose".split())
+            ), list(ds.keys())
             buckets = ds.get("buckets", spec.get("buckets", []))
             if isinstance(buckets, str):
                 buckets = [buckets]
             buckets = [expand(s) for s in buckets]
             if buckets == []:
                 buckets = [""]
-            assert len(buckets) == 1, f"{buckets}: FIXME support for multiple buckets unimplemented"
+            assert (
+                len(buckets) == 1
+            ), f"{buckets}: FIXME support for multiple buckets unimplemented"
             bucket = buckets[0]
             name = ds.get("name", "@" + bucket)
             urls = ds["shards"]
@@ -167,12 +175,16 @@ class MultiShardSample(IterableDataset):
                 urls = [urls]
             # urls = [u for url in urls for u in braceexpand.braceexpand(url)]
             urls = [
-                prefix + os.path.join(bucket, u) for url in urls for u in braceexpand.braceexpand(expand(url))
+                prefix + os.path.join(bucket, u)
+                for url in urls
+                for u in braceexpand.braceexpand(expand(url))
             ]
             resample = ds.get("resample", -1)
             nsample = ds.get("choose", -1)
             if nsample > len(urls):
-                raise ValueError(f"perepoch {nsample} must be no greater than the number of shards")
+                raise ValueError(
+                    f"perepoch {nsample} must be no greater than the number of shards"
+                )
             if (nsample > 0) and (resample > 0):
                 raise ValueError("specify only one of perepoch or choose")
             entry = MSSource(name=name, urls=urls, perepoch=nsample, resample=resample)
@@ -232,7 +244,9 @@ class ResampledShards(IterableDataset):
         self.urls = urls
         assert isinstance(self.urls[0], str)
         self.nshards = nshards
-        self.worker_seed = utils.pytorch_worker_seed if worker_seed is None else worker_seed
+        self.worker_seed = (
+            utils.pytorch_worker_seed if worker_seed is None else worker_seed
+        )
         self.deterministic = deterministic
         self.epoch = -1
 
@@ -242,7 +256,13 @@ class ResampledShards(IterableDataset):
         if self.deterministic:
             seed = utils.make_seed(self.worker_seed(), self.epoch)
         else:
-            seed = utils.make_seed(self.worker_seed(), self.epoch, os.getpid(), time.time_ns(), os.urandom(4))
+            seed = utils.make_seed(
+                self.worker_seed(),
+                self.epoch,
+                os.getpid(),
+                time.time_ns(),
+                os.urandom(4),
+            )
         if os.environ.get("WDS_SHOW_SEED", "0") == "1":
             print(f"# ResampledShards seed {seed}")
         self.rng = random.Random(seed)
