@@ -7,20 +7,14 @@
 
 """Low level iteration functions for tar archives."""
 
-import random
-import re
-import tarfile
+import random, re, tarfile
 
 import braceexpand
+
+from . import filters, gopen
 from .handlers import reraise_exception
 
-from . import gopen
-from . import filters
-
-
 trace = False
-
-
 meta_prefix = "__"
 meta_suffix = "__"
 
@@ -53,6 +47,7 @@ def valid_sample(sample):
     )
 
 
+# FIXME: UNUSED
 def shardlist(urls, *, shuffle=False):
     """Given a list of URLs, yields that list, possibly shuffled."""
     if isinstance(urls, str):
@@ -83,7 +78,7 @@ def url_opener(data, handler=reraise_exception, **kw):
                 break
 
 
-def tar_file_iterator(fileobj, skip_meta=r"__[^/]*__($|/)", handler=reraise_exception, info={}):
+def tar_file_iterator(fileobj, skip_meta=r"__[^/]*__($|/)", handler=reraise_exception):
     """Iterate over tar file, yielding filename, content pairs for the given tar stream.
 
     :param fileobj: byte stream suitable for tarfile
@@ -98,14 +93,17 @@ def tar_file_iterator(fileobj, skip_meta=r"__[^/]*__($|/)", handler=reraise_exce
                 continue
             if fname is None:
                 continue
-            if "/" not in fname and fname.startswith(meta_prefix) and fname.endswith(meta_suffix):
+            if (
+                "/" not in fname
+                and fname.startswith(meta_prefix)
+                and fname.endswith(meta_suffix)
+            ):
                 # skipping metadata for now
                 continue
             if skip_meta is not None and re.match(skip_meta, fname):
                 continue
             data = stream.extractfile(tarinfo).read()
             result = dict(fname=fname, data=data)
-            result.update(info)
             yield result
             stream.members = []
         except Exception as exn:
@@ -124,16 +122,18 @@ def tar_file_expander(data, handler=reraise_exception):
     This returns an iterator over (filename, file_contents).
     """
     for source in data:
-        info = None
+        url = source["url"]
         try:
             assert isinstance(source, dict)
             assert "stream" in source
-            info = {k: v for k, v in source.items() if k.startswith("_")}
-            for sample in tar_file_iterator(source["stream"], info=info):
-                assert isinstance(sample, dict) and "data" in sample and "fname" in sample
+            for sample in tar_file_iterator(source["stream"], handler=handler):
+                assert (
+                    isinstance(sample, dict) and "data" in sample and "fname" in sample
+                )
+                sample["__url__"] = url
                 yield sample
         except Exception as exn:
-            exn.args = exn.args + (source.get("stream"), info)
+            exn.args = exn.args + (source.get("stream"), source.get("url"))
             if handler(exn):
                 continue
             else:
@@ -150,7 +150,6 @@ def group_by_keys(data, keys=base_plus_ext, lcase=True, suffixes=None, handler=N
     for filesample in data:
         assert isinstance(filesample, dict)
         fname, value = filesample["fname"], filesample["data"]
-        info = {k: v for k, v in filesample.items() if k.startswith("__")}
         prefix, suffix = keys(fname)
         if trace:
             print(
@@ -165,15 +164,17 @@ def group_by_keys(data, keys=base_plus_ext, lcase=True, suffixes=None, handler=N
         if current_sample is None or prefix != current_sample["__key__"]:
             if valid_sample(current_sample):
                 yield current_sample
-            current_sample = dict(__key__=prefix)
-            current_sample.update(info)
+            current_sample = dict(__key__=prefix, __url__=filesample["__url__"])
         if suffix in current_sample:
-            raise ValueError(f"{fname}: duplicate file name in tar file {suffix} {current_sample.keys()}")
+            raise ValueError(
+                f"{fname}: duplicate file name in tar file {suffix} {current_sample.keys()}"
+            )
         if suffixes is None or suffix in suffixes:
             current_sample[suffix] = value
     if valid_sample(current_sample):
         yield current_sample
 
+
 def tarfile_samples(src, handler=reraise_exception):
     streams = url_opener(src, handler=handler)
     files = tar_file_expander(streams, handler=handler)
@@ -181,4 +182,4 @@ def tarfile_samples(src, handler=reraise_exception):
     return samples
 
 
-tarfile_to_samples = filters.Curried(tarfile_samples)
+tarfile_to_samples = filters.pipelinefilter(tarfile_samples)
